import curses

from betterTUI.Screen import Screen
from betterTUI.Basic import Input, Counter, Box, Button

class Form:
    def __init__(self, screen: Screen, x: int, y: int, button_label: str, inputs: dict, color=0, extra_data={}, *args):
        self.screen = screen
        self.x = x
        self.y = y
        self.button_label = button_label
        self.inputs = inputs
        self.key_ind = [x for x in inputs.keys()]
        self.content = {}
        self.pos = 0
        self.parent = None
        self.color = color

        for k, v in extra_data.items():
            self.content[k] = v

        padding = 0
        for i in inputs.values():
            if(hasattr(i, "on")):
                i.parent = self
                i.move(x, y+padding)

                padding += i.height + 1
            else:
                raise Exception("Form: inputs must have an 'on' method")

        padding = 0

        for i in inputs.values():
            if(hasattr(i, "on")):
                i.parent = self
                i.move(x, y+padding)

                padding += i.height + 1
            else:
                raise Exception("Form: inputs must have an 'on' method")

        submit_button = Button(screen, x, y+padding, button_label, color=self.color)
        submit_button.parent = self

        self.inputs["submit"] = submit_button
        self.key_ind.append("submit")

    def on(self, *args) -> int:

        while(True):
            childArgs = args
            if (self.pos > 0):
                childArgs += ("KEY_UP", )
            if (self.pos < len(self.inputs) - 1):
                childArgs += ("KEY_DOWN", )
            
            childArgs += ("\n", "KEY_ENTER", )

            key_str = self.inputs[self.key_ind[self.pos]].on(*childArgs)
            if key_str in args:
                if not(key_str == "KEY_UP" or key_str == "KEY_DOWN" or (key_str == "\n" and self.pos < len(self.inputs) - 1)):
                    return key_str
                elif(key_str == "KEY_UP" and self.pos == 0):
                    return key_str
                elif(key_str == "KEY_DOWN" and self.pos == len(self.inputs) - 1):
                    return key_str
                else:
                    self.handle_key(key_str)
            else:
                self.handle_key(key_str)

            self.screen.refresh()


    def handle_key(self, key_str):
        pos = self.pos
        if not(len(self.inputs) - 1 == pos):
            self.content[self.key_ind[self.pos]] = self.inputs[self.key_ind[self.pos]].content

        if(key_str in ["\n", "KEY_ENTER"]):
            if not (pos == len(self.inputs) - 1):
                self.pos += 1

        elif(key_str == "KEY_UP"):
            if not (pos == 0):
                self.pos -= 1

        elif(key_str == "KEY_DOWN"):
            if not (pos == len(self.inputs) - 1):
                self.pos += 1


    def delete(self):
        for input in self.inputs.values():
            input.delete()

        del self


import curses
from betterTUI.Screen import Screen

class SlimButton:
    def __init__(self, screen: Screen, x: int, y: int, label: str, color=0, *args):
        self.screen = screen
        self.x = x
        self.y = y
        self.width = len(label)
        self.height = 1
        self.label = label
        self.parent = None
        self.color = color

        self.addstr(y, x, label)

    def addstr(self, y:int, x:int, content:str, reverse=False):
        i = 0
        if (reverse): i = 1

        self.screen.attron(curses.color_pair(self.color+i))
        self.screen.addstr(y, x, content)
        self.screen.attroff(curses.color_pair(self.color+i))

    def on(self, *args) -> int:
        if(len(args) == 0):
            args =  ["\n", "KEY_ENTER"]
            
        self.addstr(self.y, self.x, self.label, True)

        while(True):
            key_str = self.screen.getkey()

            if key_str in args:
                self.addstr(self.y, self.x, self.label)
                curses.curs_set(0)
                return key_str

            self.screen.refresh()

    def move(self, x ,y):

        self.screen.addstr(self.y, self.x, " "*len(self.label))

        self.addstr(y, x, self.label)

        self.x = x
        self.y = y

    def delete(self):
        self.screen.addstr(self.y, self.x, " "*len(self.label))

        del self


#!/usr/bin/env python3

from betterTUI import *

def home(screen, child_obj=None):
    text_area = (TextArea(screen, 2, 1, 60, 15, "Text Area:", commands={"table": ["Header 1 | Header 2", "Data 1 | Data 2"], "li": ["*"], "h1": ["**"]}, content=["ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZABCDE-", "FGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ", "", "ABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZABCDEFGHIJKLMNOPQRSTUVWXYZ"]), vault)

    select = (Select(screen=screen, x=2, y=20, label="select", options=["1", "2"]), vault)

    # text_area[0].move(10, 10)
    button = (Button(screen, 14, 20, "ENTER VAULT", color=screen.COLOR_BLUE), vault)

    return [[text_area], [select], [button]]

def vault(screen, child_obj):

    searchbar = SearchBar(screen, 4, 2, 30, "Login: ", "Submit: ")

    return [[(searchbar, home)]]

def main(screen):
    curses.curs_set(0)

    screen.box()

    elements = home(screen)

    while(True):
        wrapper = Wrapper(screen, [0,0], elements)
        res = wrapper.on('ALT_Q')

        screen.clear()
        wrapper.delete()

        if(res in ['ALT_Q']): break
        else: elements = res[1](screen, res[0])
        screen.box()

Screen(main)



from betterTUI import *
from screens import *
import globals, sys, time
globals.initialize()

def main(screen):
    
    curses.curs_set(0)

    screen.timeout(1000)

    state = home(screen, None)

    last_pos = [0,0]

    while(True):

        wrapper = Wrapper(screen, last_pos, state['elements'])

        res = None
        try:
            res = wrapper.on('ALT_Q')
        except:
            res = -1

        if not(res == -1):
            screen.clear()
            wrapper.delete()

            if(res in ['ALT_Q']): break
            else: state = res[1](screen, res[0])
        else:
            state = home(screen, None)
            last_pos = wrapper.pos

try:
    Screen(main)
except Exception as e:
    sys.exit(e)


import curses

from betterTUI.Screen import Screen
from betterTUI.Basic import Input, Button

class SearchBar:
    def __init__(self, screen: Screen, x: int, y: int, width: int, input_label: str, button_label: str, color=0, *args):
        self.screen = screen
        self.x = x
        self.y = y
        self.width = width
        self.input_label = input_label
        self.content = ""
        self.pos = 0
        self.parent = None
        self.color = color

        if(width < len(button_label)+1):
            raise Exception("SearchBar: width to small!")

        self.input = Input(screen, x, y, width-(len(button_label)+1), input_label, color=self.color)
        self.input.parent = self

        self.button = Button(screen, x+width-(len(button_label)+1), y+1, button_label, color=self.color)
        self.button.parent = self

    def on(self, *args) -> int:

        while(True):
            if(self.pos == 0):
                key_str = self.input.on("KEY_RIGHT", *args)
                self.content = self.input.content
            elif(self.pos == 1):
                key_str = self.button.on("KEY_LEFT", *args)

            if key_str in args:
                if not(key_str == "KEY_RIGHT" or key_str == "KEY_LEFT"):
                    return key_str
                elif(key_str == "KEY_RIGHT" and self.pos == 1):
                    return key_str
                elif(key_str == "KEY_LEFT" and self.pos == 0):
                    return key_str
                else:
                    self.handle_key(key_str)
            else:
                self.handle_key(key_str)

            self.screen.refresh()


    def handle_key(self, key_str):
        pos = self.pos

        if(key_str == "KEY_RIGHT"):
            # TRAVEL INPUT TO RIGHT
            if not (pos == 1):
                self.pos += 1

        elif(key_str == "KEY_LEFT"):
            # TRAVEL INPUT TO LEFT
            if not (pos == 0):
                self.pos -= 1


    def delete(self):
        self.input.delete()
        self.button.delete()

        del self